/**
 * @file StereoAmp_click_PIC.c
 * @brief <h2> StereoAmp click Example </h2>
 *
 * @par
 * Example for <a href="http://www.mikroe.com">MikroElektronika's</a>
 * StereoAmp click board.
 *
 *
 *
 ******************************************************************************/

/**
 * @page        Example
 * @date        3 Nov 2016
 * @author      Branislav Marton
 * @copyright   GNU Public License
 * @version     1.0.0 - Initial testing and verification
 */

/**
 * @page TEST_CFG Test Configurations
 *
 * ### Test configuration STM : ###
 * @par
 * -<b> MCU           </b> :    P18F87K22
 * -<b> Dev. Board    </b> :    EasyPIC PRO v7 
 * -<b> Oscillator    </b> :
 * -<b> Ext. Modules  </b> :    StereoAmp click board
 * -<b> SW            </b> :    mikroC PRO for PIC v6.6.3
 *
 */

/******************************************************************************
* Includes
*******************************************************************************/
#include "StereoAmp.h"

/******************************************************************************
* Variable Definitions
*******************************************************************************/
// LCD module connections
sbit LCD_RS at RB4_bit;
sbit LCD_EN at RB5_bit;
sbit LCD_D4 at RB0_bit;
sbit LCD_D5 at RB1_bit;
sbit LCD_D6 at RB2_bit;
sbit LCD_D7 at RB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections

uint8_t volume_control_L = 0;
uint8_t volume_control_R = 0;

uint8_t volume_temp = 0;

const char custom_char_1[] = {16,16,16,16,16,16,16,16};
const char custom_char_2[] = {24,24,24,24,24,24,24,24};
const char custom_char_3[] = {28,28,28,28,28,28,28,28};
const char custom_char_4[] = {30,30,30,30,30,30,30,30};
const char custom_char_5[] = {31,31,31,31,31,31,31,31};

/******************************************************************************
* Public Functions
*******************************************************************************/
void Generate_Custom_Chars() {
    char i;
    Lcd_Cmd(64);
    for (i = 0; i<=7; i++) Lcd_Chr_CP(custom_char_1[i]);
    Lcd_Cmd(_LCD_RETURN_HOME);

    Lcd_Cmd(72);
    for (i = 0; i<=7; i++) Lcd_Chr_CP(custom_char_2[i]);
    Lcd_Cmd(_LCD_RETURN_HOME);

    Lcd_Cmd(80);
    for (i = 0; i<=7; i++) Lcd_Chr_CP(custom_char_3[i]);
    Lcd_Cmd(_LCD_RETURN_HOME);

    Lcd_Cmd(88);
    for (i = 0; i<=7; i++) Lcd_Chr_CP(custom_char_4[i]);
    Lcd_Cmd(_LCD_RETURN_HOME);

    Lcd_Cmd(96);
    for (i = 0; i<=7; i++) Lcd_Chr_CP(custom_char_5[i]);
    Lcd_Cmd(_LCD_RETURN_HOME);
}


uint8_t Read_Volume_Step(uint8_t channel) {
    uint16_t adc_value = ADC_Get_Sample(channel);

    // Downsample to range 0 - 31.
    adc_value &= 0x0FFF;
    return adc_value / 128;
}

void Draw_Bar(unsigned row_pos, unsigned char_pos, unsigned value) {
    unsigned i;

    Lcd_Out(row_pos, char_pos, "       ");

    for(i = 0; i < 7; i++) {
        if(value / ((i + 1) * 5) > 0) {
            Lcd_Chr(row_pos, char_pos + i, 4);
        } else {
            Lcd_Chr(row_pos, char_pos + i, value % 5);
            break;
        }
    }
}

void Write_Amp_Reg(uint8_t amp_addr, uint8_t reg_addr, uint8_t reg_data) {
    delay_ms(100);
    I2C1_Start();
    I2C1_Wr(amp_addr);
    I2C1_Wr( (reg_addr & REG_ADDR_MASK) | (reg_data & DATA_MASK) );
    I2C1_Stop();
}


void Init() {
    // LCD initialisation.
    Lcd_Init();
    Lcd_Cmd(_LCD_CLEAR);                // Clear display
    Lcd_Cmd(_LCD_CURSOR_OFF);          // Cursor off

    Lcd_Chr(1,1, 'L');
    Lcd_Chr(2,1, 'R');

    Generate_Custom_Chars();

    // StereoAmp initialisation.

    // IMPORTANT: Toggle I2C lines, before initialising I2C module to enable amplifiers.
    TRISC = 0;
    LATC = 0x18;
    Delay_ms(300);
    LATC = 0x00;
    Delay_ms(300);

    I2C1_Init(I2C_CLK);
    Delay_ms(800);

    // Power On and In1.
    Write_Amp_Reg(LEFT_SPEAKER, MODE_CONTROL_REG, POWER_ON | INPUT1);
    Write_Amp_Reg(RIGHT_SPEAKER, MODE_CONTROL_REG, POWER_ON | INPUT1);

    // Volume min.
    Write_Amp_Reg(LEFT_SPEAKER, VOLUME_CONTROL1_REG, 0);
    Write_Amp_Reg(RIGHT_SPEAKER, VOLUME_CONTROL1_REG, 0);

    // FAULT.
    Write_Amp_Reg(LEFT_SPEAKER, FAULT_DETECTION_REG, OUTPUT_OPEN | OUTPUT_SHT | RAIL_SHT | OVF | TSD);
    Write_Amp_Reg(RIGHT_SPEAKER, FAULT_DETECTION_REG, OUTPUT_OPEN | OUTPUT_SHT | RAIL_SHT | OVF | TSD);

    // Diagnostic Enable.
    Write_Amp_Reg(LEFT_SPEAKER, DIAGNOSTIC_CONTROL_REG, DG_EN | DG_RESET | ILIMIT);
    Write_Amp_Reg(RIGHT_SPEAKER, DIAGNOSTIC_CONTROL_REG, DG_EN | DG_RESET | ILIMIT);

    // Initialise ADC for volume control potentiometers.
    ANSEL1_bit = 1;     // Configure AN1 as analog
    ANSEL6_bit = 1;     // Configure AN6 as analog

    TRISA1_bit = 1;       // Pin RA1 is input
    TRISF1_bit = 1;       // Pin RF1 is input

    ADC_Init();
}


void Set_Volume(char amp_addr, char value) {
    Write_Amp_Reg(amp_addr, VOLUME_CONTROL1_REG, value);
}


void main() {
    Init();

    while(1) {
        volume_temp = Read_Volume_Step(LEFT_CONTROL);
        if(volume_temp != volume_control_L) {
            volume_control_L = volume_temp;
            Draw_Bar(2, 10, volume_control_L);
            Lcd_Out(2, 2, gain_text[volume_control_L]);
            Set_Volume(LEFT_SPEAKER, volume_control_L);
        }


        volume_temp = Read_Volume_Step(RIGHT_CONTROL);
        if(volume_temp != volume_control_R) {
            volume_control_R = volume_temp;
            Draw_Bar(1, 10, volume_control_R);
            Lcd_Out(1, 2, gain_text[volume_control_R]);
            Set_Volume(RIGHT_SPEAKER, volume_control_R);
        }

        Delay_ms(POLLING_PERIOD);
    }
}
/*************** END OF FUNCTIONS *********************************************/